/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2016 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

Class
    Foam::myParabolicVelocityFvPatchVectorField

Group
    grpGenericBoundaryConditions

Description
    This boundary condition provides a velocity parabolic profile condition,
    calculated as:

        \f[
			U_{patch} = U_{max} * ( 1.0 - (y - c)^2 / (r^2) ) 
        \f]

    where
    \vartable
        U_max  | maximum velocity magnitude (given by the user)
        y      | local coordinates (computed automatically)
        c      | center of the patch (computed automatically)
        r      | semi-span (computed automatically)
    \endtable



Usage
    \table
        Property   | Description                      			| Req'd?  	| Default
        U_max      | maximum velocity magnitude (scalar)       	| yes		| 0.0 
        n	       | Unit vector normal to the patch (vector)	| yes		| (0 0 0)
        t	       | Unit vector perpendicular to n (vector)  	| yes		| (0 0 0)
    \endtable

    Example of the boundary condition specification:
    \verbatim
    <patchName>
    {
        type 		myParabolicVelocity;

	    maxValue 2.0;
	    n        (1 0 0);
	    y        (0 1 0);
	
	    value    (0 0 0); 
    }
    \endverbatim


SourceFiles
    myParabolicVelocityModFvPatchVectorField.C

\*---------------------------------------------------------------------------*/

#ifndef myParabolicVelocityFvPatchVectorField_H
#define myParabolicVelocityFvPatchVectorField_H

#include "fixedValueFvPatchFields.H"
#include "Function1.H"

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

namespace Foam
{

/*---------------------------------------------------------------------------*\
     Class myParabolicVelocityFvPatchVectorField Declaration
\*---------------------------------------------------------------------------*/

class myParabolicVelocityFvPatchVectorField
:
    public fixedValueFvPatchVectorField
{
    // Private data

        //- Single valued scalar quantity, e.g. a coefficient
        scalar maxValue_;

        //- Single valued Type quantity, e.g. reference pressure pRefValue_
        //  Other options include vector, tensor
        vector n_;
        vector y_;



    // Private Member Functions

        //- Return current time
        scalar t() const;


public:

    //- Runtime type information
    TypeName("myParabolicVelocity");


    // Constructors

        //- Construct from patch and internal field
        myParabolicVelocityFvPatchVectorField
        (
            const fvPatch&,
            const DimensionedField<vector, volMesh>&
        );

        //- Construct from patch, internal field and dictionary
        myParabolicVelocityFvPatchVectorField
        (
            const fvPatch&,
            const DimensionedField<vector, volMesh>&,
            const dictionary&
        );

        //- Construct by mapping given fixedValueTypeFvPatchField
        //  onto a new patch
        myParabolicVelocityFvPatchVectorField
        (
            const myParabolicVelocityFvPatchVectorField&,
            const fvPatch&,
            const DimensionedField<vector, volMesh>&,
            const fvPatchFieldMapper&
        );

        //- Construct as copy
        myParabolicVelocityFvPatchVectorField
        (
            const myParabolicVelocityFvPatchVectorField&
        );

        //- Construct and return a clone
        virtual tmp<fvPatchVectorField> clone() const
        {
            return tmp<fvPatchVectorField>
            (
                new myParabolicVelocityFvPatchVectorField(*this)
            );
        }

        //- Construct as copy setting internal field reference
        myParabolicVelocityFvPatchVectorField
        (
            const myParabolicVelocityFvPatchVectorField&,
            const DimensionedField<vector, volMesh>&
        );

        //- Construct and return a clone setting internal field reference
        virtual tmp<fvPatchVectorField> clone
        (
            const DimensionedField<vector, volMesh>& iF
        ) const
        {
            return tmp<fvPatchVectorField>
            (
                new myParabolicVelocityFvPatchVectorField
                (
                    *this,
                    iF
                )
            );
        }


    // Member functions

        // Mapping functions

            //- Map (and resize as needed) from self given a mapping object
            virtual void autoMap
            (
                const fvPatchFieldMapper&
            );

            //- Reverse map the given fvPatchField onto this fvPatchField
            virtual void rmap
            (
                const fvPatchVectorField&,
                const labelList&
            );


        // Evaluation functions

            //- Update the coefficients associated with the patch field
            virtual void updateCoeffs();


        //- Write
        virtual void write(Ostream&) const;
};


// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //

} // End namespace Foam

// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //


#endif

// ************************************************************************* //
