/*---------------------------------------------------------------------------*\
  =========                 |
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox
   \\    /   O peration     |
    \\  /    A nd           | Copyright (C) 2016 OpenFOAM Foundation
     \\/     M anipulation  |
-------------------------------------------------------------------------------
License
    This file is part of OpenFOAM.

    OpenFOAM is free software: you can redistribute it and/or modify it
    under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    OpenFOAM is distributed in the hope that it will be useful, but WITHOUT
    ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
    FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
    for more details.

    You should have received a copy of the GNU General Public License
    along with OpenFOAM.  If not, see <http://www.gnu.org/licenses/>.

\*---------------------------------------------------------------------------*/

#include "myParaboloidFvPatchVectorField.H"
#include "addToRunTimeSelectionTable.H"
#include "fvPatchFieldMapper.H"
#include "volFields.H"
#include "surfaceFields.H"

// * * * * * * * * * * * * * Private Member Functions  * * * * * * * * * * * //



// * * * * * * * * * * * * * * * * Constructors  * * * * * * * * * * * * * * //

Foam::myParaboloidFvPatchVectorField::
myParaboloidFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchVectorField(p, iF),

    //these are the default values

    //max value of the profile
    maxValue_(1),
    //max value of radius
    radius_(1),

    //direction normal to the patch
    //n_(1, 0, 0),
    n_(Zero),

    //directions in the 2d plane of the patch they must be orthogonal
    //y_(0, 1, 0),    
    //y1_(0, 0, 1),
    y_(Zero),
    y1_(Zero)



{
}


Foam::myParaboloidFvPatchVectorField::
myParaboloidFvPatchVectorField
(
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const dictionary& dict
)
:
    fixedValueFvPatchVectorField(p, iF),

    maxValue_(readScalar(dict.lookup("maxValue"))),
    radius_(readScalar(dict.lookup("radius"))),
    n_(pTraits<vector>(dict.lookup("n"))),
    y_(pTraits<vector>(dict.lookup("y"))),
   //this is new input in the dictionary for reading the 2d directions in the patch plane and the radius of the patch
    y1_(pTraits<vector>(dict.lookup("y1")))

{

    if (mag(n_) < SMALL || mag(y_) < SMALL)
    {
        FatalErrorIn("myParaboloidFvPatchVectorFielddict)")
            << "n or y given with zero size not correct"
            << abort(FatalError);
    }

    n_ /= mag(n_);
    y_ /= mag(y_);
    y1_ /= mag(y1_);

    fixedValueFvPatchVectorField::evaluate();

    /*
    //Initialise with the value entry if evaluation is not possible
    fvPatchVectorField::operator=
    (
        vectorField("value", dict, p.size())
    );
    */
}


Foam::myParaboloidFvPatchVectorField::
myParaboloidFvPatchVectorField
(
    const myParaboloidFvPatchVectorField& ptf,
    const fvPatch& p,
    const DimensionedField<vector, volMesh>& iF,
    const fvPatchFieldMapper& mapper
)
:
    fixedValueFvPatchVectorField(ptf, p, iF, mapper),
    maxValue_(ptf.maxValue_),
    radius_(ptf.maxValue_),
    n_(ptf.n_), 
    y_(ptf.y_), 
    y1_(ptf.y1_)

{}


Foam::myParaboloidFvPatchVectorField::
myParaboloidFvPatchVectorField
(
    const myParaboloidFvPatchVectorField& ptf
)
:
    fixedValueFvPatchVectorField(ptf),
    maxValue_(ptf.maxValue_),
    radius_(ptf.maxValue_),
    n_(ptf.n_), 
    y_(ptf.y_), 
    y1_(ptf.y1_)

{}


Foam::myParaboloidFvPatchVectorField::
myParaboloidFvPatchVectorField
(
    const myParaboloidFvPatchVectorField& ptf,
    const DimensionedField<vector, volMesh>& iF
)
:
    fixedValueFvPatchVectorField(ptf, iF),
    maxValue_(ptf.maxValue_),
    radius_(ptf.maxValue_),
    n_(ptf.n_), 
    y_(ptf.y_), 
    y1_(ptf.y1_)

{}


// * * * * * * * * * * * * * * * Member Functions  * * * * * * * * * * * * * //

void Foam::myParaboloidFvPatchVectorField::autoMap
(
    const fvPatchFieldMapper& m
)
{
    fixedValueFvPatchVectorField::autoMap(m);

}


void Foam::myParaboloidFvPatchVectorField::rmap
(
    const fvPatchVectorField& ptf,
    const labelList& addr
)
{
    fixedValueFvPatchVectorField::rmap(ptf, addr);

    const myParaboloidFvPatchVectorField& tiptf =
        refCast<const myParaboloidFvPatchVectorField>(ptf);

;
}


void Foam::myParaboloidFvPatchVectorField::updateCoeffs()
{
    if (updated())
    {
        return;
    }


    //here we find the min and max coordinates of the bounding box enclosing the patch
    boundBox bb(patch().patch().localPoints(), true);

    //here we find the center coordinates, assuming it is symmetrical
    vector ctr = 0.5*(bb.max() + bb.min());

    //here we get the radius in the patch plane, assuming it is symmetrical
    //vector compradius = (bb.max()) - ctr;

    //cell centers in patch
    const vectorField& c = patch().Cf();

	//This was for the parabolicd profile
    //scalarField coord = 2*((c - ctr) & y_)/((bb.max() - bb.min()) & y_);

    //here we calculate the 2d coordinates of the paraboloid/circle in the patch plane
    scalarField coord2 = ((c - ctr) & y_)*((c - ctr) & y_) + ((c - ctr) & y1_)*((c - ctr) & y1_) - radius_;


    fixedValueFvPatchVectorField::operator==
    (
	    n_*maxValue_*-4*(coord2)
	    //n_*maxValue_*1*(coord2)/max(coord2)
        //n_*maxValue_*(1.0 - sqr(coord2))
    );


    fixedValueFvPatchVectorField::updateCoeffs();

    word write_outputs = "no";		//yes or no  -  OpenFOAM equivalent to to char write_outputs[] = "yes";

    if (write_outputs == "yes")
	{
		Info << endl << "Face centers (c):" << endl;
		Info << c << endl; 

		Info << endl << "Patch center (ctr):" << endl;
		Info << ctr << endl; 

		Info << endl << "Patch (c - ctr):" << endl;
		Info << c - ctr << endl;

		Info << endl << "Patch ((c - ctr) & y_):" << endl;
		Info << ((c - ctr) & y_) << endl;

		Info << endl << "Patch max bound (bb.max):" << endl;
		Info <<  bb.max() << endl;

		Info << endl << "Patch min bound (bb.max):" << endl;
		Info << bb.min() << endl;

		Info << endl << "Patch ( (bb.max() - bb.min()) & y_ ):" << endl;
		Info << ((bb.max() - bb.min()) & y_) << endl;

		Info << endl << "Patch coord:" << endl;
		Info << coord2 << endl;

		Info << endl << "Patch ( n_*maxValue_*-4*(coord2) ) :" << endl;
		Info << n_*maxValue_*-4*(coord2) << endl;

		Info << endl << "Loop for c, operator, operator.x()" << endl;
		forAll(c, faceI)
		{
		    Info << c[faceI] << "     " << n_*maxValue_*-4*(coord2) << endl;
		}

		Info << endl << endl;
	}

}


void Foam::myParaboloidFvPatchVectorField::write
(
    Ostream& os
) const
{
    fvPatchVectorField::write(os);
    os.writeKeyword("maxValue") << maxValue_ << token::END_STATEMENT << nl;
    os.writeKeyword("n") << n_ << token::END_STATEMENT << nl;
    os.writeKeyword("y") << y_ << token::END_STATEMENT << nl;
    os.writeKeyword("y1")<< y1_ << token::END_STATEMENT << nl;
    os.writeKeyword("radius")<< radius_ << token::END_STATEMENT << nl;
    //writeEntry(os, "value");				//not needed anymore to write visualize field
}


// * * * * * * * * * * * * * * Build Macro Function  * * * * * * * * * * * * //

namespace Foam
{
    makePatchTypeField
    (
        fvPatchVectorField,
        myParaboloidFvPatchVectorField
    );
}

// ************************************************************************* //
